@@ -149,7 +149,7 @@ public:
 	size_t nsymbols() const;
         bool get_first_symbol(ex &x) const;
         symbolset symbols() const;
-	ex op(size_t i) const { return bp->op(i); }
+	const ex op(size_t i) const { return bp->op(i); }
 	ex sorted_op(size_t i) const;
 	ex operator[](const ex & index) const { return (*bp)[index]; }
 	ex operator[](size_t i) const { return (*bp)[i]; }
@@ -203,7 +203,8 @@ public:
 	ex series(const ex & r, int order, unsigned options = 0) const;
 
 	// rational functions
-	ex normal(int level = 0) const;
+	ex normal(int level = 0, bool noexpand_combined=false,
+                        bool noexpand_numer=true) const;
 	ex to_rational(exmap & repl) const;
 	ex to_rational(lst & repl_lst) const;
 	ex to_polynomial(exmap & repl) const;
@@ -266,6 +267,11 @@ private:
 	static ptr<basic> construct_from_string_and_lst(const std::string &s, const ex &l);
 	void makewriteable();
 	void share(const ex & other) const;
+        static ex deep_combine_fractions(ex e);
+        struct combine_map_function : public map_function {
+                ex operator()(const ex & e) override { return deep_combine_fractions(e); }
+        };
+
 
 // member variables
 
@@ -721,8 +727,9 @@ inline ex denom(const ex & thisex)
 inline ex numer_denom(const ex & thisex)
 { return thisex.numer_denom(); }
 
-inline ex normal(const ex & thisex, int level=0)
-{ return thisex.normal(level); }
+inline ex normal(const ex & thisex, int level=0, bool noexpand_combined=false,
+                bool noexpand_numer=true)
+{ return thisex.normal(level, noexpand_combined, noexpand_numer); }
 
 inline ex to_rational(const ex & thisex, lst & repl_lst)
 { return thisex.to_rational(repl_lst); }
